C******************************************************************************
C PROGRAM MOCACYL (MONTE CARLO FOR CYLINDER) DETERMINES HEAT FLUXES WITHIN A GENERAL,
C AXISYMMETRIC MEDIUM WITH KNOWN TEMPERATURE AND PROPERTY DISTRIBUTIONS.
C This version uses energy partitioning (gradual absorption of bundle energy)
C RUNNING THIS CODE REQUIRES:
C -- USER-SUPPLIED SUBROUTINE NAMED 'PROPS' AND FUNCTION 'ABSCO'
C -- AN INPUT FILE NAMED 'datlam.dat'
C
C SUBROUTINE PROPS(R,Z,T,SVF,PCO2,PH2O,PAC,PSC) PROVIDES LOCAL (R,Z) CONDITIONS FOR THE MEDIUM
C  INPUT:
C   R     = RADIAL COORDINATE (CM)
C   Z     = AXIAL COORDINATE (CM)
C  OUTPUT:
C   T     = TEMPERATURE (K)
C   SVF   = SOOT VOLUME FRACTION
C   PCO2  = CO2 PARTIAL PRESSURE (BAR)
C   PH2O  = H2O PARTIAL PRESSURE (BAR)
C   PAC   = PARTICLE ABSORPTION COEFFICIENT (1/CM)
C   PSC   = PARTICLE SCATTERING COEFFICIENT (1/CM)
C  NOTE: THIS ROUTINE REQUIRES THE COMMON STATEMENT 'COMMON RL,ZL,AN,STN,STK,NRR,NZL,NRN'
C
C FUNCTION ABSCO(SVF,PCO2,PH2O,PAC,W,T)  CALCULATES THE LOCAL, SPECTRAL ABSORPTION 
C COEFFICIENT AS FUNCTION OF LOCAL CONDITIONS; 
C  INPUT:
C   SVF   = SOOT VOLUME FRACTION
C   PCO2  = CO2 PARTIAL PRESSURE (BAR)
C   PH2O  = H2O PARTIAL PRESSURE (BAR)
C   PAC   = PARTICLE ABSORPTION COEFFICIENT (1/CM)
C   W     = WAVELENGTH (MICRO-M)
C   T     = TEMPERATURE (K)
C  OUTPUT:
C   ABSCO = LOCAL, SPECTRAL ABSORPTION COEFFICIENT (IN 1/CM)
C  NOTES: THIS ROUTINE REQUIRES THE COMMON STATEMENT 'COMMON RL,ZL,AN,STN,STK,NRR,NZL,NRN'
C         FOR THE CASE W<0 THE ROUTINE SHOULD RETURN A GRAY ABSORPTION COEFFICIENT
C
C FILE 'datlam.dat' CONTAINS RANDOM NUMBER RELATIONSHIPS AND IS GENERATED BY PROGRAM 'RNARRAY'
C NOTE: RNARRAY MUST BE RUN WITH IDENTICAL VALUES FOR (NRP,NZP,NNP,RL,ZL,AN,STN,STK,IGRAY) AS
C       SPECIFIED IN THIS PROGRAM
C
C MODIFIED VERSION FOR RADIAL VARIATIONS JULY 3, 1981
C MODIFIED TO INCLUDE NON-BLACK WALLS JULY 12, 1981
C MODIFIED TO INCLUDE ISOTROPIC SCATTERING JULY 14,1981
C MODIFIED TO INCLUDE WALL EMISSION JULY 15, 1981
C THIS VERSION USES GRADUAL ATTENUATION BY ABSORPTION
C### UPDATED FOR VAX/VMS AUGUST 6, 1987 ###
C UPDATED APRIL 2003
C
C******************************************************************************
C INPUT:
C NRP   = NO OF RADIAL NODES FOR MEDIUM EMISSION RANDOM NUMBER RELATIONSHIPS
C NZP   = NO OF AXIAL NODES FOR MEDIUM EMISSION RANDOM NUMBER RELATIONSHIPS
C NNP   = NO OF RANDOM NUMBERS FOR MEDIUM EMISSION RELATIONSHIPS
C NR    = NO OF RADIAL NODES FOR SURFACE FLUX CALCULATIONS
C NZ    = NO OF AXIAL NODES FOR SURFACE FLUX CALCULATIONS
C T3(NZ)= TEMPERATURE OF LINER WALL (r=R), (K)
C EPS(3)= SURFACE EMITTANCES: EPS(1)=INLET, EPS(2)=EXIT, EPS(3)=LINER
C RL    = RADIUS OF CYLINDER, (CM)
C ZL    = LENGTH OF CYLINDER, (CM)
C TW0   = SURFACE TEMPERATURE OF INLET
C TWL   = SURFACE TEMPERATURE OF EXIT
C AN    = REFRACTIVE INDEX OF MEDIUM (AN=1.0 FOR GASES)
C STN   = REFRACTIVE INDEX OF SOOT
C STK   = ABSORPTIVE INDEX OF SOOT
C NTOTAL= TOTAL NUMBER OF PHOTON BUNDLES EMITTED FROM MEDIUM (NUMBER OF BUNDLES 
C         FOR SURFACE EMISSION ARE CHOSEN AUTOMATICALLY AS FUNCTION OF NTOTAL)
C IGRAY = GRAY/NONGRAY MEDIUM SWITCH: IGRAY=0 NONGRAY, 
C         IGRAY=1 GRAY (IGNORING CONTRIBUTION FROM GASES; ABS.COEFF=PAC)
C IWALL = WALL EMISSION SWITCH: IWALL=0 ONLY CONSIDER MEDIUM EMISSION;
C         IWALL=1 ALSO CONSIDER SURFACE EMISSION 
C LU    = LOGICAL UNIT NUMBER FOR OUTPUT: LU=6 SENDS OUTPUT TO SCREEN,
C         OTHER (LEGAL) VALUES SEND OUTPUT TO FILE results.dat
C******************************************************************************
C OUTPUT:
C PRINTOUT OF RELEVANT INPUT DATA, PLUS
C QW(1,1..NR) = AXIAL RADIATIVE HEAT FLUX FOR NR RADIAL NODES AT INLET (W/CM2)
C QW(2,1..NR) = AXIAL RADIATIVE HEAT FLUX FOR NR RADIAL NODES AT EXIT (W/CM2)
C QW(3,1..NZ) = RADIAL RADIATIVE HEAT FLUX FOR NZ AXIAL NODES AT LINER (W/CM2)
C NOTE; QW>0 DENOTES FLUX INTO WALL, QW<0 FLUX OUT OF WALL
C******************************************************************************
C LINES ENCLOSED IN *'S ARE TO BE USED ONLY IF DETAILED RAY
C TRACING IS DESIRED; OTHERWISE THEY SHOULD HAVE A 'C' IN
C THE FIRST COLUMN
C*********************************************************
      PROGRAM MOCAC
      PARAMETER (NRP=21,NZP=21,NNP=101,NR=5,NZ=7,NS=MAX(NRP,NZP))
      REAL A(3,NS),EW(3,NS),QW(3,NS),WVE2(NRP,NZP,NNP),ZR(NRP,NNP),
     1      RRA(NNP),PLMCL(NZP),ZX(NZP),ZRW(NNP),EPS(3),ETOTAL(4),T3(NZ)
      INTEGER NTOT(4)
      COMMON RL,ZL,AN,STN,STK,NRR,NZL,NRN
C
C GENERAL DATA
C
C T3=liner temperature (K), EPS=emittances(inlet,exit,liner),
C RL=radius (cm), ZL=length (cm), AN=refractive index of medium (-), 
C STN= soot refractive index, STK= soot absorptive index, 
C NRR=# radial nodes, NZL=# axial nodes, NRN=# wavelength nodes
C
       DATA T3/7*1000./
       DATA EPS/1.,1.,.5/
       RL=10.
       ZL=10. 
       TW0=1000.
       TWL=0.
       AN=1.
       STN=1.89
       STK=0.92
       NTOTAL=100000
       IGRAY=1
       IWALL=0
       LU=7
C
       NRR=NRP
       NZL=NZP
       NRN=NNP-1
       NRNP1=NRN+1
       DR=RL/(NRR-1.)
       DZ=ZL/(NZL-1.)
C
C UNIVERSAL CONSTANTS       
       PI=ATAN(1.)*4.
       SIG=5.670E-12
       C2=14388.
C
C READING OF ARRAYS R=F(RAN-#), Z=F(R,RAN-#) & LAM=F(R,Z,RAN-#)
C FROM DISK FILE, ALSO CENTER-LINE K-PLANCK & EM. TOTAL ENERGY
C
       IF(LU.NE.6) OPEN(UNIT=LU,FILE='results.dat',STATUS='UNKNOWN')
       OPEN(UNIT=3,FILE='datlam.dat',STATUS='UNKNOWN')
       READ(3,3) ETOTAL(1)
       READ(3,3) (PLMCL(J),J=1,NZL)
       READ(3,3) (RRA(J),J=1,NRNP1)
       DO 110 K=1,NRR
         READ(3,3) (ZR(K,J),J=1,NRNP1)
         IF(IGRAY.EQ.1) GOTO 110
         DO 100 I=1,NZL
            READ(3,3) (WVE2(K,I,J),J=1,NRNP1)
    3       FORMAT(1X,5E13.6)
  100    CONTINUE
  110  CONTINUE
       IF(IWALL.EQ.0) GOTO 15
C CALCULATION OF ARRAY ZW=F(RAN-#) FOR WALL EMISSION
       CALL ZLOC(T3,NZ,ZRW,NNP,ETOTAL(4))
   15  CONTINUE  
   
C IDLE RUN OF RNUM TO CREATE DIFFERENT SETS OF STATISTICAL SAMPLES
       IDLE=1+NTOTAL/100
       DO 4 N=1,IDLE
    4  RN=RNUM(DSEED)
       DRS=RL/NR
       DZS=ZL/NZ
C
C INITIALIZATION FOR AREAS AND ENERGIES
C
       DO 5 I=1,NR
          QW(1,I)=-IWALL*EPS(1)*AN*AN*SIG*TW0**4
          QW(2,I)=-IWALL*EPS(2)*AN*AN*SIG*TWL**4
          A(1,I)=2.*PI*(I-.5)*DRS**2
   5   A(2,I)=A(1,I)
       DO 6 I=1,NZ
          ZX(I)=(I-.5)*DZS
          QW(3,I)=-IWALL*EPS(3)*AN*AN*SIG*T3(I)**4
   6   A(3,I)=2.*PI*RL*DZS
       ETOTAL(2)=-PI*RL*RL*QW(1,1)
       ETOTAL(3)=-PI*RL*RL*QW(2,1)
       ETOTAL(4)=ETOTAL(4)*EPS(3)
C******************************************************************
C      WRITE(LU,910)
 910   FORMAT(2X,'AFTER WALL EMISSION:')
C      WRITE(LU,500) (I,ZX(I),QW(3,I),I=1,NZ)
C******************************************************************
C
C CYCLE FOR BUNDLE EMISSION BEGINS
C
       ICYCL=0
       NTOT(1)=NTOTAL
  160  ICYCL=ICYCL+1
       NBUNDL=0
       DO 7 K=1,3
       DO 7 J=1,NZ
   7   EW(K,J)=0.   
   
       GOTO(10,12,14,16,170),ICYCL
C ICYCL=1:  EMISSION FROM MEDIUM
   10  RN1=RNUM(DSEED)
       RN2=RNUM(DSEED)
       R2=NRN*RN2+1.
       I2=R2
       RR=RRA(I2)+(R2-I2)*(RRA(I2+1)-RRA(I2))
       R1=NRN*RN1+1.
       I1=R1
       R3=(NRR-1)*RR/RL+1.
       I3=R3
       ZD=ZR(I3,I1)+(R1-I1)*(ZR(I3,I1+1)-ZR(I3,I1))
       ZH=ZR(I3+1,I1)+(R1-I1)*(ZR(I3+1,I1+1)-ZR(I3+1,I1))
       ZZ=ZD+(R3-I3)*(ZH-ZD)
       IEM=1
C*************************************************************
C      WRITE(LU,900)
 900   FORMAT(3X,'EMISSION FROM MEDIUM')
C*************************************************************
       GOTO 18                 
       
C ICYCL=2:  EMISSION FROM INLET WALL
   12  RR=RL*SQRT(RNUM(DSEED))
       ZZ=0.
       IEM=2
C*************************************************************
C      WRITE(LU,901)
 901   FORMAT(3X,'EMISSION FROM INLET WALL')
C*************************************************************
       GOTO 18         
       
C ICYCL=3:  EMISSION FROM EXIT WALL
   14  RR=RL*SQRT(RNUM(DSEED))
       ZZ=ZL
       IEM=3
C*************************************************************
C      WRITE(LU,902)
 902   FORMAT(3X,'EMISSION FROM EXIT WALL')
C*************************************************************
       GOTO 18  
       
C ICYCL=4:  EMISSION FROM LINER
   16  RR=RL
       R1=NRN*RNUM(DSEED)+1.
       I1=R1
       ZZ=ZRW(I1)+(R1-I1)*(ZRW(I1+1)-ZRW(I1))
       IEM=4
C**************************************************************
C      WRITE(LU,903)
 903   FORMAT(3X,'EMISSION FROM LINER')
C**************************************************************
   18  CONTINUE
   
       NBUNDL=NBUNDL+1
C CHOOSE WAVELENGTH OF EMISSION
       IF(IGRAY.NE.1 .AND. ICYCL.EQ.1) THEN
          RN5=RNUM(DSEED)
          AN5=NRN*RN5+1
          IN5=AN5
          ANM=AN5-IN5
          R5=ZZ/ZL*(NZL-1.)+1.
          I5=R5
          I4=I3+1
          I6=I5+1
          WLLL=WVE2(I3,I5,IN5)+ANM*(WVE2(I3,I5,IN5+1)-WVE2(I3,I5,IN5))
          WLLH=WVE2(I3,I6,IN5)+ANM*(WVE2(I3,I6,IN5+1)-WVE2(I3,I6,IN5))
          WLHL=WVE2(I4,I5,IN5)+ANM*(WVE2(I4,I5,IN5+1)-WVE2(I4,I5,IN5))
          WLHH=WVE2(I4,I6,IN5)+ANM*(WVE2(I4,I6,IN5+1)-WVE2(I4,I6,IN5))
          WLL=WLLL+(R5-I5)*(WLLH-WLLL)
          WLH=WLHL+(R5-I5)*(WLHH-WLHL)
          WL=WLL+(R3-I3)*(WLH-WLL)
       ELSE
          WL=-1.
       ENDIF
       EBUNDL=1.
C CHOOSE SCATTERING PATHLENGTH
       SABS=0.
  200  SSCATR=-ALOG(RNUM(DSEED))
C CHOOSE EMISSION/REFLECTION OR SCATTERING DIRECTION
  150  RN3=RNUM(DSEED)
       RN4=RNUM(DSEED)
       PHI=2.*PI*RN3
       GOTO(210,220,220,230),IEM
C EMISSION/REFLECTION FROM MEDIUM
  210  COSB=1.-2.*RN4
       GOTO 240
C EMISSION/REFLECTION FROM INLET OR EXIT WALL
  220  COSB=(-1)**IEM*SQRT(RN4)
       GOTO 240
C EMISSION/REFLECTION FROM LINER
  230  COSBP=SQRT(RN4)
       SINBP=SQRT(1.-RN4)
       COSPP=COS(PHI)
       COSB=SINBP*COSPP
       SINB=SQRT(1.-COSB*COSB)+1.E-6
       COSP=-COSBP/SINB
       SINP=SQRT(1.-COSP*COSP)
       GOTO 250
  240  SINB=SQRT(1.-COSB*COSB)
       COSP=COS(PHI)
       SINP=SIN(PHI)
  250  CONTINUE
C****************************************************************
C      BET=180./PI*ATAN(SINB/COSB)
C      IF(BET.LT.0.) BET=180.+BET
C      IF(IEM.EQ.4) PHI=ATAN(COSP/SINP)
C      PHID=180./PI*PHI
C      WRITE(LU,904) NBUNDL,RR,ZZ,WL,BET,PHID,EBUNDL,SSCATR
 904   FORMAT(6X,'B#=',I3,' R=',F4.1,' Z=',F5.1,' WL=',F4.1,
     1 ' BET=',F4.0,' PHI=',F4.0,' E=',F4.2,' SSCATR=',F5.2)
C****************************************************************
C DISTANCE ALONG RAY TO LINER WALL
       S=(SQRT(RL*RL-(RR*SINP)**2)-RR*COSP)/SINB
       ZE=ZZ+S*COSB
       IF(ZE.GT.0.) GOTO 20
       IS=1
       ZE=0.
       GOTO 30
   20  IF(ZE.LT.ZL) GOTO 40
       IS=2
       ZE=ZL
   30  S=(ZE-ZZ)/COSB
       RE=SQRT(RR*RR+2.*S*RR*SINB*COSP+(S*SINB)**2)
       T3E=RE/RL
       IJE=NR*T3E+1.
       GOTO 50
   40  T1E=ZE/ZL
       IJE=NZ*T1E+1.
       IS=3
       RE=RL
C OPTICAL DISTANCE TO POINT OF IMPACT OR SCATTERING
   50  NSC=S/DZS
       NSC=NSC+1/(NSC+1)
       DELS=S/(2.*NSC)
       SSCAT=0.
       CALL PROPS(RR,ZZ,TRZ1,FRZ1,PC1,PH1,ACRZ,SCRZ)
       THREEA=ABSCO(FRZ1,PC1,PH1,ACRZ,WL,TRZ1)*DELS/3.
       THREES=SCRZ*DELS/3.
       DO 60 L=1,NSC
          ONEA=THREEA
          ONES=THREES
          SP=(2.*L-1.)*DELS
          ZP=ZZ+SP*COSB
          RP=SQRT(RR*RR+2.*SP*RR*SINB*COSP+(SP*SINB)**2)
          CALL PROPS(RP,ZP,TRZ,FRZ,PCP,PHP,ACP,SCP)
          TWOA=ABSCO(FRZ,PCP,PHP,ACP,WL,TRZ)*DELS/.75
          TWOS=SCP*DELS/.75
          SP=2.*L*DELS
          ZP=ZZ+SP*COSB
          RP=SQRT(RR*RR+2.*SP*RR*SINB*COSP+(SP*SINB)**2)
          CALL PROPS(RP,ZP,TRZ,FRZ,PCP,PHP,ACP,SCP)
          THREEA=ABSCO(FRZ,PCP,PHP,ACP,WL,TRZ)*DELS/3.
          THREES=SCP*DELS/3.
          OLDA=SABS
          SABS=SABS+ONEA+TWOA+THREEA
          OLDS=SSCAT
          SSCAT=SSCAT+ONES+TWOS+THREES
          IF(SSCAT.LT.SSCATR) GOTO 60
          FRAC=(SSCAT-SSCATR)/(SSCAT-OLDS)
          SP=2.*DELS*(L-FRAC)
          SABS=SABS-FRAC*(SABS-OLDA)
          ZZ=ZZ+SP*COSB
          RR=SQRT(RR*RR+2.*SP*RR*SINB*COSP+(SP*SINB)**2)
          IEM=1
C******************************************************************
C      WRITE(LU,905) S,SABS,SSCAT
 905   FORMAT(6X,'BUNDLE IS SCATTERED; S=',F4.1,' SABS=',F5.1,
     1 ' SSCAT=',F5.1)
C******************************************************************
          IF(SABS.GT.6.) GOTO 90
          GOTO 200
   60  CONTINUE
C ATTENUATE BUNDLE ENERGY, SHORTEN SCATTERING PATH
       EBUNDL=EBUNDL*EXP(-SABS)
       SSCATR=SSCATR-SSCAT
C*******************************************************************
C      WRITE(LU,906)IS,RE,ZE,S,SABS,SSCATR,EBUNDL
 906   FORMAT(6X,'BUNDLE HITS SURFACE',I2,'; RE=',F4.1,' ZE=',F5.1,
     1 ' S=',F4.1,' SABS=',F5.1,' SSCATR=',F5.1,' E=',F3.2)
C*******************************************************************
       SABS=0.
C
C ADD ATTENUATED BUNDLE ENERGY TO WALL NODE
C
       EW(IS,IJE)=EW(IS,IJE)+EBUNDL*EPS(IS)
C DETERMINE BUNDLE ENERGY CONTENT AND SET UP FOR REFLECTION
       EBUNDL=(1.-EPS(IS))*EBUNDL
       IF(EBUNDL.LE.0.01) THEN
          EW(IS,IJE)=EW(IS,IJE)+EBUNDL
          GOTO 90
       ENDIF
       RR=RE
       ZZ=ZE
       IEM=IS+1
       GOTO 150
   90  IF(NBUNDL.EQ.NTOT(ICYCL)) GOTO 180
       GOTO(10,12,14,16,170),ICYCL
C
C CALCULATE NODAL FLUXES
C
  180  AVEN=ETOTAL(ICYCL)/NBUNDL
       DO 70 IS=1,3
          NIS=NR*(10/(IS+8))+NZ*(IS/3)
       DO 70 I=1,NIS
   70  QW(IS,I)=QW(IS,I)+EW(IS,I)*AVEN/A(IS,I)
       IF(IWALL.EQ.0 .OR. ICYCL.EQ.4) GOTO 170
       NTOT(ICYCL+1)=NTOTAL*(ETOTAL(ICYCL+1)/ETOTAL(1))**.2+1
       NTOT(ICYCL+1)=MIN(NTOT(ICYCL+1),NTOTAL)
C******************************************************************
C      IF(ICYCL.EQ.1) WRITE(LU,911)
C      IF(ICYCL.EQ.2) WRITE(LU,912)
C      IF(ICYCL.EQ.3) WRITE(LU,913)
 911   FORMAT(/2X,'AFTER MEDIUM EMISSION:')
 912   FORMAT(/2X,'AFTER INLET WALL EMISSION:')
 913   FORMAT(/2X,'AFTER EXIT WALL EMISSION:')
C      WRITE(LU,500) (I,ZX(I),QW(3,I),I=1,NZ)
C******************************************************************
       GOTO 160
C
  170  WRITE(LU,400) RL,ZL,(EPS(N),N=1,3),AN,NTOT
       WRITE(LU,410) (I,I=1,NZ)
       WRITE(LU,415) TW0,TWL,(T3(I),I=1,NZ)
       WRITE(LU,405) 
       DO 80 I=1,NZL
          ZZ=(I-1.)*ZL/(NZL-1.)
          CALL PROPS(0.,ZZ,TZ,FRZ,PCO2Z,PH2OZ,ACZ,SCZ)
   80  WRITE(LU,420) I,ZZ,TZ,FRZ,PCO2Z,PH2OZ,PLMCL(I),ACZ,SCZ
       WRITE(LU,500) (I,ZX(I),QW(3,I),I=1,NZ)
       WRITE(LU,520) (J,J=1,NR)
       WRITE(LU,530) (QW(1,J),J=1,NR)
       WRITE(LU,540) (QW(2,J),J=1,NR)
  400  FORMAT(10X,12HGENERAL DATA/10X,12H************//
     1    15X,30HBURNER RADIUS                 ,F6.2,3H CM/
     2    15X,30HBURNER LENGTH                 ,F6.2,3H CM/
     X    15X,30HEMITTANCES:             INLET:,F6.2/
     Y    15X,30H                         EXIT:,F6.2/
     Z    15X,30H                        LINER:,F6.2/
     A    15X,30HREFRACTIVE INDEX              ,F6.2/
     3    15X,26HNUMBER OF BUNDLES: MEDIUM:,I7/
     X    15X,26H                    INLET:,I7/
     Y    15X,26H                     EXIT:,I7/
     Z    15X,26H                    LINER:,I7/)
  410  FORMAT(//5X,'WALL TEMPERATURES (DEG.K)'//7X,'TW0   TWL  T3:I=',
     1 I2,20I6)
  415  FORMAT(5X,2F6.0,4X,21F6.0)
  405  FORMAT(///5X,34HPROPERTY VALUES ALONG CENTER LINE://
     4 10X,'I     Z    T      FR   PCO2   PH2O    K-PL  ABSC   SCAT'/
     5 10X,'     CM   DEG.K    %    ATM    ATM    CM-1  CM-1   CM-1'/)
  420  FORMAT(7X,I4,F7.1,F7.0,F6.2,5F7.3)
  500  FORMAT(//5X,'RADIAL HEAT FLUXES AT NODES I (W/SQCM)'//
     1    10X,15HI     Z   QWALL/15X,11HCM   W/SQCM//(8X,I3,2F7.1))
  520  FORMAT(//5X,'AXIAL HEAT FLUXES AT NODES J (W/SQCM)'//
     1 '  J  ',21I6/)
  530  FORMAT('  Q0  ',21F6.1)
  540  FORMAT('  QL  ',21F6.1)
       STOP
       END
C **************************************************************************
      SUBROUTINE ZLOC(T3,NZ,ZRW,NNP,ETOT)
      REAL T3(NZ),ZRW(NNP),Z(NZ+1),ZINT(NZ+1),B(NZ),C(NZ),D(NZ)
      COMMON RL,ZL,AN,STN,STK,NRR,NZL,NRN
      TWOPISIGMA=3.5626E-11
C CALCULATE INTEGRAL_0^L (sigma*T^4 2piRdz)
      ZINT(1)=0.
      Z(1)=0.
      DZ=ZL/NZ
      DO I=2,NZ+1
         Z(I)=(I-1.)*DZ
         ZINT(I)=ZINT(I-1)+TWOPISIGMA*RL*DZ*T3(I-1)**4
      ENDDO
      ETOT=ZINT(NZ+1)
C NORMALIZE VALUES
      DO I=1,NZ+1
         ZINT(I)=ZINT(I)/ETOT
      ENDDO
C fit monotonically increasing spline through inverse function
      CALL SPLMI(NZ,ZINT,Z,B,C,D)
C evaluate  z(random-#) for equidistant random-#
      ZRW(1)=0.
      JZ=1
      DO 80 IZ=1,NRN
         RZ=IZ/FLOAT(NRN)
   60    IF(ZINT(JZ).GE.RZ) GOTO 70
         JZ=JZ+1
         GOTO 60
   70    DELZ=RZ-ZINT(JZ-1)
   80 ZRW(IZ+1)=Z(JZ-1)+((D(JZ-1)*DELZ+C(JZ-1))*DELZ+B(JZ-1))*DELZ      
      RETURN
      END
